/*
 * Windows H19 Terminal Emulator
 * 
 * Written by William S. Hall
 *	      3665 Benton Street, #66
 *	      Santa Clara, CA 95051
 *
 * Basic Emulator Menu Module
 */

MENUITEM "&Online", IDM_ONLINE
POPUP "E&mulator"
BEGIN
    MENUITEM "Set &Communications...", IDM_COMM
    MENUITEM "Set &Terminal...", IDM_TERM
    MENUITEM "&Clear Communications Port", IDM_CLEARCOM
    MENUITEM "&Reset Terminal", IDM_RESET
END
POPUP "&Edit"
BEGIN
    MENUITEM "&Function Keys...", IDM_SPECIALKEYS
    MENUITEM "&Slap Screen", IDM_COPY
    MENUITEM "&Paste", IDM_PASTE, GRAYED
END
