NAME

DESCRIPTION 'Windows H19 Terminal Emulator'

STUB 'Winstub.exe'

HEAPSIZE	1024
STACKSIZE	4096

CODE	MOVEABLE
DATA	MOVEABLE MULTIPLE

SEGMENTS
    _WINH19RS PRELOAD MOVEABLE DISCARDABLE
    _WINH19TM PRELOAD MOVEABLE DISCARDABLE
    _WINH19NT PRELOAD MOVEABLE DISCARDABLE
    _WINH19NR LOADONCALL MOVEABLE DISCARDABLE

EXPORTS
    MainWndProc		@1
    MainWndSubclassProc	@2
    AboutBoxProc	@3
    TermWndProc		@4
    StatWndProc		@5
    SetCommParams	@6
    SetTermParams	@7
    SetStringParams	@8
