/*
 * Windows H19 Terminal Emulator
 * 
 * Written by William S. Hall
 *	      3665 Benton Street, #66
 *	      Santa Clara, CA 95051
 *
 * Basic Emulator Resource Module
 */

DT_ABOUT dialog 22,17,154,98
STYLE WS_POPUP | WS_DLGFRAME
BEGIN
    CTEXT "Microsoft Windows"			-1,0,5,154,8
#if defined(KERMIT)
    CTEXT "H-19 Terminal Emulator with Kermit"	-1,0,17,154,8
#else
    CTEXT "Heath H-19 Terminal Emulator"	-1,0,17,154,8
#endif
    CTEXT "Version 4.11, 09/24/89"		-1,25,29,104,8
    CTEXT "Copyright (c) 1988, 1989 by"		-1,0,41,154,8
    CTEXT "William S. Hall"			-1,0,53,154,8
    CTEXT "3665 Benton Street, #66"		-1,0,61,154,8
    CTEXT "Santa Clara, CA 95051, USA"		-1,0,69,154,8
    DEFPUSHBUTTON "OK"				IDOK,61,81,32,12,WS_GROUP
END

Winh19 ACCELERATORS
BEGIN
    VK_F1, IDM_SF1, VIRTKEY, SHIFT
    VK_F1, IDM_CF1, VIRTKEY,CONTROL
    VK_F1, IDM_CSF1, VIRTKEY,SHIFT,CONTROL

    VK_F2, IDM_SF2, VIRTKEY, SHIFT
    VK_F2, IDM_CF2, VIRTKEY,CONTROL
    VK_F2, IDM_CSF2, VIRTKEY,SHIFT,CONTROL

    VK_F3, IDM_SF3, VIRTKEY, SHIFT
    VK_F3, IDM_CF3, VIRTKEY,CONTROL
    VK_F3, IDM_CSF3, VIRTKEY,SHIFT,CONTROL

    VK_F4, IDM_SF4, VIRTKEY, SHIFT
    VK_F4, IDM_CF4, VIRTKEY,CONTROL
    VK_F4, IDM_CSF4, VIRTKEY,SHIFT,CONTROL

    VK_F5, IDM_SF5, VIRTKEY, SHIFT
    VK_F5, IDM_CF5, VIRTKEY,CONTROL
    VK_F5, IDM_CSF5, VIRTKEY,SHIFT,CONTROL

    VK_F7, IDM_SF7, VIRTKEY, SHIFT
    VK_F7, IDM_CF7, VIRTKEY,CONTROL
    VK_F7, IDM_CSF7, VIRTKEY,SHIFT,CONTROL

    VK_F8, IDM_SF8, VIRTKEY, SHIFT
    VK_F8, IDM_CF8, VIRTKEY,CONTROL
    VK_F8, IDM_CSF8, VIRTKEY,SHIFT,CONTROL

    VK_F9, IDM_SF9, VIRTKEY, SHIFT
    VK_F9, IDM_CF9, VIRTKEY,CONTROL
    VK_F9, IDM_CSF9, VIRTKEY,SHIFT,CONTROL

    VK_F10, IDM_F10, VIRTKEY
    VK_F10, IDM_SF10, VIRTKEY, SHIFT
    VK_F10, IDM_CF10, VIRTKEY,CONTROL
    VK_F10, IDM_CSF10, VIRTKEY,SHIFT,CONTROL
    VK_F11, IDM_F11, VIRTKEY
    VK_F11, IDM_SF11, VIRTKEY, SHIFT
    VK_F11, IDM_CF11, VIRTKEY,CONTROL
    VK_F11, IDM_CSF11, VIRTKEY,SHIFT,CONTROL
    VK_F12, IDM_F12, VIRTKEY
    VK_F12, IDM_SF12, VIRTKEY, SHIFT
    VK_F12, IDM_CF12, VIRTKEY,CONTROL
    VK_F12, IDM_CSF12, VIRTKEY,SHIFT,CONTROL
END

STRINGTABLE
BEGIN
    IDS_APPNAME,	"Winh19"
    IDS_ABOUT,		"A&bout..."
#if defined(KERMIT)
    IDS_WINTITLE	"H-19 Emulator with Kermit on "
#else
    IDS_WINTITLE,	"H-19 Terminal Emulator on "
#endif
    IDS_OFFLINE		"&Offline"
    IDS_ONLINE		"&Online"
    IDS_DISPLAYNAME	"DISPLAY"
    IDS_NOCOMOPEN	"Cannot open any communications port."
    IDS_NOCOMSET	"Cannot set communications parameters."
    IDS_ADDDEFAULTS	"Add default entries to WIN.INI?"
    IDS_CONTINUEPROG	"Win.ini modified; continue with program?"
    IDS_CANNOTCLOSECOM	"Cannot close communications port."
    IDS_CANNOTCREATETERM "Cannot create terminal window."
    IDS_CANNOTCREATESTATUS "Cannot create status window."
    IDS_CANNOTCHANGEPORT "Cannot change communications port."
    IDS_FIRSTPORT	"port1"
    IDS_SECONDPORT	"port2"
    IDS_COM1		"COM1:"
    IDS_COM2		"COM2:"
    IDS_COMMSTR		"1200,n,8,1"
    
    IDS_BLOCKCURSOR	"BlockCursor"
    IDS_KEYCLICK	"KeyClick"
    IDS_WRAPAROUND	"WrapAround"
    IDS_AUTOLFONCR	"AutoLFonCR"
    IDS_AUTOCRONLF	"AutoCRonLF"
    IDS_ANSIMODE	"ANSIMode"
    IDS_SHIFTEDKEYPAD	"ShiftedKeypad"
    IDS_FULLDUPLEX	"FullDuplex"

    IDS_TERMNAME		"WinH19Term"
    IDS_STATNAME		"WinH19Stat"
    IDS_TERMINAL		"Heath 19"

    IDS_SPKEYID1		"F1"
    IDS_SPKEYID2		"F2"
    IDS_SPKEYID3		"F3"
    IDS_SPKEYID4		"F4"
    IDS_SPKEYID5		"F5"
    IDS_SPKEYID6		"F6"
    IDS_SPKEYID7		"F7"
    IDS_SPKEYID8		"F8"
    IDS_SPKEYID9		"F9"
    IDS_SPKEYID10		"F10"
    IDS_SPKEYID11		"F11"
    IDS_SPKEYID12		"F12"

    IDS_SSPKEYID1		"SF1"
    IDS_SSPKEYID2		"SF2"
    IDS_SSPKEYID3		"SF3"
    IDS_SSPKEYID4		"SF4"
    IDS_SSPKEYID5		"SF5"
    IDS_SSPKEYID6		"SF6"
    IDS_SSPKEYID7		"SF7"
    IDS_SSPKEYID8		"SF8"
    IDS_SSPKEYID9		"SF9"
    IDS_SSPKEYID10		"SF10"
    IDS_SSPKEYID11		"SF11"
    IDS_SSPKEYID12		"SF12"

    IDS_CSPKEYID1		"CF1"
    IDS_CSPKEYID2		"CF2"
    IDS_CSPKEYID3		"CF3"
    IDS_CSPKEYID4		"CF4"
    IDS_CSPKEYID5		"CF5"
    IDS_CSPKEYID6		"CF6"
    IDS_CSPKEYID7		"CF7"
    IDS_CSPKEYID8		"CF8"
    IDS_CSPKEYID9		"CF9"
    IDS_CSPKEYID10		"CF10"
    IDS_CSPKEYID11		"CF11"
    IDS_CSPKEYID12		"CF12"

    IDS_CSSPKEYID1		"CSF1"
    IDS_CSSPKEYID2		"CSF2"
    IDS_CSSPKEYID3		"CSF3"
    IDS_CSSPKEYID4		"CSF4"
    IDS_CSSPKEYID5		"CSF5"
    IDS_CSSPKEYID6		"CSF6"
    IDS_CSSPKEYID7		"CSF7"
    IDS_CSSPKEYID8		"CSF8"
    IDS_CSSPKEYID9		"CSF9"
    IDS_CSSPKEYID10		"CSF10"
    IDS_CSSPKEYID11		"CSF11"
    IDS_CSSPKEYID12		"CSF12"

    IDS_LINES			"lines"
    IDS_COLUMNS			"columns"
END

DT_COMM dialog 12,20,174,152
STYLE WS_POPUP  | WS_DLGFRAME
begin
    groupbox	"Baud Rate"	-1,4,4,70,128
    groupbox	"Word Size"	-1,78,4,48,40
    groupbox	"Port"		-1,78,48,48,56
    groupbox	"Parity"	-1,130,4,40,86
    edittext			IDD_BAUDRATE,26,16,24,12, WS_TABSTOP | WS_GROUP
    radiobutton	" &110",	IDD_110,8,32,32,12,WS_TABSTOP | WS_GROUP
    radiobutton	" 150",		IDD_150,40,32,32,12
    radiobutton	" 300",		IDD_300,8,46,32,12
    radiobutton	" 600",		IDD_600,40,46,32,12
    radiobutton	"1200",		IDD_1200,8,60,32,12
    radiobutton	"1800",		IDD_1800,40,60,32,12
    radiobutton	"2000",		IDD_2000,8,74,32,12
    radiobutton	"2400",		IDD_2400,40,74,32,12
    radiobutton	"3600",		IDD_3600,8,88,32,12
    radiobutton	"4800",		IDD_4800,40,88,32,12
    radiobutton	"7200",		IDD_7200,8,102,32,12
    radiobutton	"9600",		IDD_9600,40,102,32,12
    radiobutton	"19200",	IDD_19200,8,116,32,12
    radiobutton	"Other",	IDD_OTHER,40,116,32,12

    radiobutton	"&7 bits"	IDD_7BIT,82,16,36,12,WS_TABSTOP | WS_GROUP
    radiobutton	"8 bits"	IDD_8BIT,82,30,36,12

    radiobutton	"C&OM1:"	IDD_COM1,82,60,32,12,WS_TABSTOP | WS_GROUP
    radiobutton	"COM2:"		IDD_COM2,82,74,32,12
    checkbox	"&Default?"	IDD_DEFPORT,82,88,42,12,WS_TABSTOP | WS_GROUP

    radiobutton	"&None"		IDD_NONE,134,18,32,12,WS_TABSTOP | WS_GROUP
    radiobutton	"Odd"		IDD_ODD,134,32,32,12
    radiobutton	"Even"		IDD_EVEN,134,46,32,12
    radiobutton	"Mark"		IDD_MARK,134,60,32,12
    radiobutton	"Space"		IDD_SPACE,134,74,32,12

    checkbox	"&SAVE SETTINGS?" IDD_SAVE,90,116,68,12,WS_TABSTOP | WS_GROUP
    defpushbutton "&Ok",	IDOK,32,136,32,14,WS_TABSTOP | WS_GROUP
    pushbutton    "&Cancel",	IDCANCEL,102,136,32,12,WS_TABSTOP | WS_GROUP
end

DT_TERM dialog 12,20,168,110
STYLE WS_POPUP  | WS_DLGFRAME
BEGIN
    groupbox	"Echo"		-1,4,4,44,40
    groupbox	"Mode"		-1,4,48,44,40
    groupbox	"Keypad",	-1,52,4,44,40
    groupbox	"Cursor",	-1,52,48,44,40

    radiobutton	"&Remote",	IDD_FULL,8,16,36,12,WS_GROUP | WS_TABSTOP
    radiobutton	"Local",	IDD_HALF,8,28,32,12

    radiobutton	"ANSI",		IDD_ANSI,8,60,32,12,WS_GROUP | WS_TABSTOP
    radiobutton	"&Heath"	IDD_HEATH,8,72,32,12

    radiobutton	"&Normal"	IDD_NORMAL,56,16,36,12,WS_GROUP | WS_TABSTOP
    radiobutton	"Shifted"	IDD_SHIFTED,56,28,38,12

    radiobutton	"&Block",	  IDD_BLOCK,56,60,32,12,WS_GROUP | WS_TABSTOP
    radiobutton	"Line"		  IDD_LINE,56,72,32,12

    checkbox	"Line &Wrap"	  IDD_WRAP,100,6,60,12,WS_GROUP | WS_TABSTOP
    checkbox	"&Auto LF on CR"  IDD_LFCR,100,20,64,12,WS_GROUP | WS_TABSTOP
    checkbox	"A&uto CR on LF"  IDD_CRLF,100,34,64,12,WS_GROUP | WS_TABSTOP
    checkbox	"&SAVE SETTINGS?" IDD_SAVE,100,62,76,12,WS_GROUP | WS_TABSTOP

    defpushbutton "&Ok",	IDOK,24,92,40,14,WS_GROUP | WS_TABSTOP
    pushbutton    "&Cancel",	IDCANCEL,104,92,40,14, WS_TABSTOP
END

DT_STRING dialog 4,20,304,82
STYLE WS_POPUP  | WS_DLGFRAME
BEGIN
    ltext	"String:"	-1,4,6,28,8
    edittext			IDD_STRINGEDIT,36,4,264,12,WS_GROUP | WS_TABSTOP
    radiobutton	"F1"		IDD_F1,4,24,24,12,WS_GROUP | WS_TABSTOP
    radiobutton	"F2"		IDD_F2,28,24,24,12
    radiobutton	"F3"		IDD_F3,52,24,24,12
    radiobutton	"F4"		IDD_F4,76,24,24,12
    radiobutton	"F5"		IDD_F5,100,24,24,12
    radiobutton	"F6"		IDD_F6,124,24,24,12,WS_DISABLED
    radiobutton	"F7"		IDD_F7,148,24,24,12
    radiobutton	"F8"		IDD_F8,172,24,24,12
    radiobutton	"F9"		IDD_F9,196,24,24,12
    radiobutton	"F10"		IDD_F10,220,24,24,12
    radiobutton	"F11"		IDD_F11,248,24,24,12
    radiobutton	"F12"		IDD_F12,276,24,24,12
    radiobutton	"None"		IDD_NOSHIFT,36,44,48,12,WS_GROUP | WS_TABSTOP
    radiobutton	"Shift"		IDD_SHIFT,96,44,48,12
    radiobutton	"Control"	IDD_CONTROL,156,44,48,12
    radiobutton	"Control Shift"	IDD_CTRLSHIFT,216,44,60,12
    defpushbutton "Save",	IDOK,80,64,40,14,WS_GROUP | WS_TABSTOP
    pushbutton    "Quit",	IDCANCEL,194,64,40,14, WS_TABSTOP
END
