/*
 * Windows Kermit
 * 
 * Written by William S. Hall
 *	      3665 Benton Street, #66
 *	      Santa Clara, CA 95051
 *
 * functions shared between emulator and protocol modules
 */

void krmKermitDispatch(HWND, BYTE *, int);
void krmProcessKermitMenu(HWND hWnd, WORD menuitem);
BOOL krmInit(HWND hWnd, HANDLE hInstance);
BOOL FAR PASCAL WinIniCallBack(HWND hWnd,LONG lParam);
void BroadcastWinIniChange(void);
void krmAdjustHeight(short, short);
void krmShowTransferData(HWND hWnd, HDC hDC, BOOL iconic);
BOOL NEAR SearchKey(char *str, char *key, int len);
