/*
 * Windows Kermit
 * 
 * Written by William S. Hall
 *	      3665 Benton Street, #66
 *	      Santa Clara, CA 95051
 *
 * Kermit specific resources
 */

#define TAB	WS_TABSTOP
#define GRP	WS_GROUP
#define TABGRP	WS_TABSTOP | WS_GROUP

STRINGTABLE
BEGIN
    KRM_USERCNX			"Transaction aborted by user"
    KRM_COMPLETE		"Transaction complete"
    KRM_ERROR_PKT		"Error packet received"
    KRM_NO_SEND_FILE		"No file to send"
    KRM_FILE_OPEN_ERROR		"Cannot open file"
    KRM_TERM_OPEN_ERROR		"Cannot display data"
    KRM_FILE_WRITE_ERROR	"Cannot write file"
    KRM_FILE_CLOSE_ERROR	"Cannot close file"
    KRM_FILE_READ_ERROR		"Cannot read file"
    KRM_UNKNOWN_PACKET		"Unknown packet type"
    KRM_RETRY_LIMIT		"Retry limit exceeded"
END

STRINGTABLE
BEGIN
    IDS_KRM_KERMIT		"Kermit"
    IDS_KRM_ADDDEFAULTS		"Add Kermit Defaults?"
    IDS_KRM_TIMER		"Timer"
    IDS_KRM_FILEWARN		"Filewarn"
    IDS_KRM_DISCARD		"Discard"
    IDS_KRM_BELL		"Bell"
    IDS_KRM_GETFILES		"Get filespec..."
    IDS_KRM_CWD			"Change to directory..."
    IDS_KRM_PASSWORD		"[Password]"
    IDS_KRM_REMOTEDIR		"Directory [filespec...]"
    IDS_KRM_REMOTETYPE		"Type filespec..."
    IDS_KRM_REMOTEHELP		"Help [about...]"
    IDS_KRM_REMOTEDEL		"Delete filespec..."
    IDS_KRM_REMOTEWHO		"Who [user(s)...]"
    IDS_KRM_REMOTESPACE		"Space [user(s)...]"
    IDS_KRM_REMOTEHOST		"Host command..."
END

DT_KRM_XFER dialog 16,32,152,76
STYLE WS_CHILD  | WS_BORDER | WS_VISIBLE
CAPTION "Kermit Transfer Data"
begin
    rtext	"Receiving file:",  IDD_OPERATION,4,4,60,8
    ltext	"",		    IDD_FILENAME,68,4,80,8,WS_BORDER
    rtext	"Saved as:",      -1,4,16,60,8
    ltext	"",		    IDD_SAVENAME,68,16,50,8,WS_BORDER
    rtext	"Bytes moved:",   -1,4,28,60,8
    ltext	"0",		    IDD_BYTESMOVED,68,28,34,8,WS_BORDER
    rtext	"Total packets:", -1,4,40,60,8
    ltext	"0"		    IDD_PACKETS,68,40,22,8,WS_BORDER
    rtext	"Total retries:", -1,4,52,60,8
    ltext	"0",		    IDD_RETRIES,68,52,22,8,WS_BORDER
    rtext	"Percentage:",     -1,4,64,60,8
    ltext	"",		    IDD_PERCENTAGE,68,64,14,8,WS_BORDER
end

DT_KRM_GETFILE dialog 30,30,144,56
STYLE WS_POPUP  | WS_DLGFRAME
begin
    ctext		"",		IDD_STATICTEXT1	0,4,144,8
    edittext		IDD_GETNAME,	4,16,136,12, ES_AUTOHSCROLL | TABGRP
    defpushbutton 	"OK",		IDOK,18,36,50,14,TABGRP
    pushbutton    	"Cancel",	IDCANCEL, 76,36,50,14,TAB
end

DT_KRM_REMOTE2 dialog 30,30,144,84
STYLE WS_POPUP  | WS_DLGFRAME
begin
    ctext		""		IDD_STATICTEXT1	0,4,144,8
    ctext		"",		IDD_STATICTEXT2,4,32,136,8
    edittext		IDD_GETTEXT1,	4,16,136,12,ES_AUTOHSCROLL | TABGRP
    edittext		IDD_GETTEXT2,	4,44,136,12,ES_AUTOHSCROLL | TABGRP
    defpushbutton 	"OK",		IDOK,18,64,50,14,TABGRP
    pushbutton    	"Cancel",	IDCANCEL,	76,64,50,14,TAB
end

DT_KRM_MXPARAMS dialog 20,20,120,110
STYLE WS_POPUP | WS_DLGFRAME
BEGIN
    ctext	"Set Miscellaneous Parameters"	-1,0,4,120,8
    checkbox	"Timeout on"			IDD_TIMER,4,18,52,12,TABGRP
    checkbox	"Duplicate filename warning"	IDD_FILEWARN,4,30,116,12,TAB
    checkbox	"Discard incomplete file"	IDD_FILEDISCARD,4,42,104,12,TAB
    checkbox	"Bell at end of transfer"	IDD_KRM_BELL,4,54,104,12,TAB
    checkbox	"Save Settings?"		IDD_KRM_SAVE,26,72,68,12,TABGRP
    DEFPUSHBUTTON "OK"				IDOK,14,88,32,14,TABGRP
    PUSHBUTTON	"CANCEL",			IDCANCEL,74,88,32,14,TAB
END

DT_KRM_SENDFILE dialog 20,20,180,108
STYLE WS_POPUP | WS_DLGFRAME
BEGIN
    LTEXT "Search specification:"	-1,4,6,84,8
    CTEXT ""		IDD_PATHDISPLAY,68,28,108,8
    EDITTEXT		IDD_EDITSELECT,92,4,84,12,ES_AUTOHSCROLL | TABGRP
    LISTBOX		IDD_LISTSELECT,4,24,60,80,LBS_MULTIPLESEL | TABGRP
    LISTBOX		IDD_PATHSELECT,116,40,60,64, TABGRP
    DEFPUSHBUTTON "SELECT"	IDOK,74,44,32,14, TABGRP
    PUSHBUTTON	  "SEND"	IDD_SEND,74,64,32,14, TABGRP
    PUSHBUTTON	  "CANCEL",	IDCANCEL,74,84,32,14, TAB
END
