/*
 * Windows Kermit
 * 
 * Written by William S. Hall
 *	      3665 Benton Street, #66
 *	      Santa Clara, CA 95051
 *
 * Kermit specific menu items
 */

POPUP "&Protocol"
BEGIN
    MENUITEM "&Set..", IDM_MXPARAMS
END
POPUP "&Files"
BEGIN
    MENUITEM "&Send...", IDM_KRM_SEND
    MENUITEM "&Receive", IDM_KRM_RECEIVE
    MENUITEM "&Get...", IDM_KRM_GET
    MENUITEM Separator
    MENUITEM "E&xclude file", IDM_CANCELFILE, Grayed
    MENUITEM "&Zap batch", IDM_CANCELBATCH, Grayed
    MENUITEM "&Error abort", IDM_ERRORCANCEL, Grayed
    MENUITEM "&Cancel protocol", IDM_CANCELPROTOCOL, Grayed
END
POPUP "&Remote"
BEGIN
    MENUITEM "&Chdir...", IDM_KRM_CWD
    MENUITEM "&Directory...",IDM_KRM_REMOTEDIR
    Menuitem "&Type...",IDM_KRM_REMOTETYPE
    Menuitem "D&elete...",IDM_KRM_REMOTEDEL
    MENUITEM SEPARATOR
    Menuitem "&Who...", IDM_KRM_REMOTEWHO
    Menuitem "&Space...",IDM_KRM_REMOTESPACE
    MENUITEM "&Help...", IDM_KRM_REMOTEHELP,MENUBARBREAK
    MENUITEM SEPARATOR
    Menuitem "H&ost Command...",IDM_KRM_REMOTEHOST
    MENUITEM SEPARATOR
    MENUITEM "&Finish", IDM_KRM_FINISH
    MENUITEM "&Logout", IDM_KRM_LOGOUT
    MENUITEM "&Bye", IDM_KRM_BYE
END
