
SEGMENTS
    _WINKRMKM LOADONCALL MOVEABLE DISCARDABLE
    _WINKRMDG LOADONCALL MOVEABLE DISCARDABLE
    _WINKRMFX LOADONCALL MOVEABLE DISCARDABLE

EXPORTS
    GetFileList		@100
    SendFileDlgProc	@101
    SetMiscParams	@102
    krmDoTimeout	@103
    krmShowChildren	@104
    krmRemoteChdir	@105
    krmRemoteCmdDlgBox	@106
    krmXferDlgBox	@107

